unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Shellapi, Menus, ExtCtrls, Buttons, IniFiles,
  Clipbrd, Printers, About, StdCtrls;

const
  MaxColumn = 6;

  Captions : array[0..MaxColumn] of string =
    ('Name', 'DOS Name', 'Size', 'Created',
     'Modified', 'Accessed', 'Attributes');

  DefaultWidths : array[0..MaxColumn] of integer =
    (220, 100, 100, 124, 124, 124, 70);

  DefaultChecked : array[0..MaxColumn] of boolean =
    (true, false, false, false, true, false, false);

type
  TDest = (ToClipboard, ToPrinter, ToFile);

type
  TMyListView = class(TListView)
    procedure DefaultHandler(var Message); override;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure ColumnClick(Sender: TObject; Column: TListColumn);
    procedure Compare(Sender: TObject; Item1,
      Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure LVMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LVKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    FilenameList : TStringList;
    ColumnLists : array[1..MaxColumn] of TStringList;
    ColumnWidths : array[0..MaxColumn] of integer;
    SortCol : integer;
    ReverseSort : boolean;
    SortType : integer;
  public
  end;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Panel1: TPanel;
    Exit1: TMenuItem;
    ImageList1: TImageList;
    SaveAs1: TMenuItem;
    Print1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    Edit1: TMenuItem;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Clear1: TMenuItem;
    SelectAll1: TMenuItem;
    View1: TMenuItem;
    FullPath1: TMenuItem;
    CreationDate1: TMenuItem;
    ModifiedDate1: TMenuItem;
    AccessedDate1: TMenuItem;
    A1: TMenuItem;
    MSDOSName1: TMenuItem;
    Size1: TMenuItem;
    Help1: TMenuItem;
    HelpTopics1: TMenuItem;
    AboutFileGrab1: TMenuItem;
    Image1: TImage;
    SaveAsSpeedButton: TSpeedButton;
    PrintSpeedButton: TSpeedButton;
    CutSpeedButton: TSpeedButton;
    CopySpeedButton: TSpeedButton;
    ClearSpeedButton: TSpeedButton;
    FontDialog1: TFontDialog;
    PrintDialog1: TPrintDialog;
    PrintFont1: TMenuItem;
    SaveDialog1: TSaveDialog;
    Label1: TLabel;
    ScreenFont1: TMenuItem;
    N3: TMenuItem;
    Paste1: TMenuItem;
    procedure CalcBuffer;
    procedure MakeBuffer;
    procedure CopyList(Dest : TDest);
    procedure ReadIni;
    procedure WriteIni;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure SaveColumnWidths;
    procedure ColumnsUpdate;
    procedure ToggleChecked(Sender: TObject);
    procedure AboutFileGrab1Click(Sender: TObject);
    procedure HelpTopics1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure SaveAsSpeedButtonClick(Sender: TObject);
    procedure PrintSpeedButtonClick(Sender: TObject);
    procedure CutSpeedButtonClick(Sender: TObject);
    procedure CopySpeedButtonClick(Sender: TObject);
    procedure ClearSpeedButtonClick(Sender: TObject);
    procedure SelectAllSpeedButtonClick(Sender: TObject);
    procedure ScreenFont1Click(Sender: TObject);
    procedure PrintFont1Click(Sender: TObject);
    procedure UpdateButtons;
    procedure Edit1Click(Sender: TObject);
    procedure Paste1Click(Sender: TObject);

  private
    { Private declarations }
    MyListView : TMyListView;
    FieldLen : array[0..MaxColumn] of integer;
    Buffer : PChar;
    BufferSize : longint;
    PrintName : string;
    PrintSize : integer;
    PrintStyle : TFontStyles;
    Busy : boolean;
    CommaDelimited : boolean;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

var
  InternalPasteMessage : integer;

{$R *.DFM}

constructor TMyListView.Create(AOwner : TComponent);
var
  i : integer;
begin
  inherited Create(AOwner);
  OnColumnClick:= ColumnClick;
  OnCompare:= Compare;
  OnMouseUp:= LVMouseUp;
  OnKeyUp:= LVKeyUp;
  FilenameList:= TStringList.Create;
  for i:= 1 to MaxColumn do
    ColumnLists[i]:= TStringList.Create;
end;

destructor TMyListView.Destroy;
var
  i : integer;
begin
  for i:= Items.Count - 1 downto 0 do
    TStringList(Items[i].Data).Free;

  inherited;
  FilenameList.Free;
  for i:= 1 to MaxColumn do
    ColumnLists[i].Free;
end;

procedure TMyListView.ColumnClick(Sender: TObject;
  Column: TListColumn);
var
  i, j : integer;
begin
  if SortCol = Column.Index then
    ReverseSort:= not ReverseSort
  else
    ReverseSort:= false;
  SortCol:= Column.Index;
  SortType:= 0;
  j:= 0;
  if SortCol > 0 then
    for i:= 1 to MaxColumn do
      if MainForm.View1.Items[i].Checked then
      begin
        inc(j);
        if j = SortCol then
          SortType:= i;
      end;
  AlphaSort;
end;

procedure TMyListView.Compare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);

  function CompareDate(const S1, S2 : string) : Integer;
  var
    T1, T2 : TDateTime;
  begin
    T1:= StrToDateTime(S1);
    T2:= StrToDateTime(S2);
    if T1 < T2 then Result:= -1
    else if T1 > T2 then Result:= 1
    else Result:= 0;
  end;

  function CompareNumber(const S1, S2 : string) : Integer;
  var
    T1, T2 : longint;
  begin
    T1:= StrToInt(S1);
    T2:= StrToInt(S2);
    if T1 < T2 then Result:= -1
    else if T1 > T2 then Result:= 1
    else Result:= 0;
  end;

begin
  case SortType of
    0    : Compare:= AnsiCompareStr(Item1.Caption, Item2.Caption);
    1, 6 : Compare:= AnsiCompareStr(Item1.SubItems[SortCol - 1],
             Item2.SubItems[SortCol - 1]);
    2    : Compare:= CompareNumber(Item1.SubItems[SortCol - 1],
             Item2.SubItems[SortCol - 1]);
    3..5 : Compare:= CompareDate(Item1.SubItems[SortCol - 1],
             Item2.SubItems[SortCol - 1]);
  end;
  if ReverseSort then
    Compare:= -Compare;
end;

function GetDate(const T : TFileTime; OldT : Integer) : TDateTime;
var
  LocTime : TFileTime;
  SysTime : TSystemTime;
begin
  if (T.dwLowDateTime = 0) and (T.dwHighDateTime = 0) then
    Result:= FileDateToDateTime(OldT)
  else
    begin
      FileTimeToLocalFileTime(T, LocTime);
      FileTimeToSystemTime(LocTime, SysTime);
      with SysTime do
       Result:= EncodeTime(wHour, wMinute, wSecond, wMilliseconds) +
                EncodeDate(wYear, wMonth, wDay);
    end;
end;

function AttributeString(Attribute : dword) : string;
var
  A, H, S, R : char;
begin
  if Attribute and faArchive = faArchive then
    A:= 'A'
  else
    A:= ' ';
  if Attribute and faHidden = faHidden then
    H:= 'H'
  else
    H:= ' ';
  if Attribute and faSysFile = faSysFile then
    S:= 'S'
  else
    S:= ' ';
  if Attribute and faReadOnly = faReadOnly then
    R:= 'R'
  else
    R:= ' ';
  Result:= A + H + S + R;
end;

procedure TMyListView.DefaultHandler(var Message);
var
  DroppedFilesCnt : integer;
  DropIndex : integer;
  NewFilesStart, NewIndex : Integer;
  BufferStr : string;
  Buffer : PChar;
  BufferSize : integer;
  ImageIndex: integer;
  TS : TSearchRec;
  DTFormat : string;
  TSF : TSHFileInfo;
begin
  inherited DefaultHandler(Message);
  with TMessage(Message) do
  if (Msg = WM_DROPFILES) or
     (Msg = InternalPasteMessage) then
  begin
    NewFilesStart := FilenameList.Count;
    DroppedFilesCnt:= DragQueryFile(wParam, -1, PChar(0), 0);
    for DropIndex:= 0 to DroppedFilesCnt - 1 do
    begin
      BufferSize:= DragQueryFile(wParam, DropIndex, PChar(0), 0);
      GetMem(Buffer, BufferSize + 1);
      try
        DragQueryFile(wParam, DropIndex, Buffer, BufferSize + 1);
        with FilenameList do
          if IndexOf(Buffer) = -1 then
            AddObject(Buffer, Pointer(0));
      finally
        FreeMem(Buffer);
      end;
    end;
    if Msg = WM_DROPFILES then DragFinish(wParam);

    for NewIndex:= NewFilesStart to FilenameList.Count-1 do
    begin
      BufferStr:= FilenameList[NewIndex];
      FillChar(TSF, SizeOf(TSF), 0);

      if SHGetFileInfo(PChar(BufferStr), 0, TSF, SizeOf(TSF),
        SHGFI_ICON or SHGFI_SMALLICON) <> 0 then
      begin
        with MainForm.Image1 do
        begin
          Canvas.Brush.Color:= clWindow;
          Canvas.FillRect(ClientRect);
          DrawIconEx(Canvas.Handle, 0, 0, TSF.hIcon,
            16, 16, 0, 0, DI_DEFAULTSIZE or DI_NORMAL);

          ImageIndex:= MainForm.ImageList1.Add(
            Picture.Bitmap, Picture.Bitmap);
        end;
        FilenameList.Objects[NewIndex]:= Pointer(ImageIndex);
      end;

      if FindFirst(PChar(BufferStr), faReadOnly or faHidden or
        faSysFile or faDirectory, TS) = 0 then
      with TS.FindData do
      begin
        if cAlternateFileName = '' then
          ColumnLists[1].Add(cFileName)
        else
          ColumnLists[1].Add(cAlternateFileName);
        ColumnLists[2].Add(IntToStr((nFileSizeHigh * MAXDWORD)
          + nFileSizeLow));
        DTFormat:= ShortDateFormat + ' ' + ShortTimeFormat;
        ColumnLists[3].Add(FormatDateTime(DTFormat,
          GetDate(ftCreationTime, TS.Time)));
        ColumnLists[4].Add(FormatDateTime(DTFormat,
          GetDate(ftLastWriteTime, TS.Time)));
        ColumnLists[5].Add(FormatDateTime(DTFormat,
          GetDate(ftLastAccessTime, TS.Time)));
        ColumnLists[6].Add(AttributeString(dwFileAttributes));
      end;
    end;
    MainForm.SaveColumnWidths;
    MainForm.ColumnsUpdate;
  end;
end;

procedure TMyListView.LVMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MainForm.UpdateButtons;
end;

procedure TMyListView.LVKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  MainForm.UpdateButtons;
end;

procedure TMainForm.CalcBuffer;
var
  i, j, k : longint;
begin
  for i:= 0 to MaxColumn do
    FieldLen[i]:= 0;

  with MyListView do
  begin
    for i:= 0 to Items.Count - 1 do
      if (Items[i].Selected) and
         (length(Items[i].Caption) > FieldLen[0]) then
        FieldLen[0]:= length(Items[i].Caption);

    k:= 0;
    for i:= 1 to MaxColumn do
      if View1.Items[i].Checked then
      begin
        for j:= 0 to Items.Count - 1 do
          if (Items[j].Selected) and
             (length(Items[j].SubItems[k]) > FieldLen[i]) then
            FieldLen[i]:= length(Items[j].SubItems[k]);
        inc(k);
      end;

    k:= FieldLen[0];
    for i:= 1 to MaxColumn do
      if View1.Items[i].Checked then
        k:= k + 1 + FieldLen[i]; {1 space between fields}
    k:= k + 2; {crlf}
    BufferSize:= k * SelCount;
  end;
end;

procedure TMainForm.MakeBuffer;
var
  i, j, k, p, q : longint;
  crlf : string;
  FieldSpacing : string;
begin
  crlf:= char(13) + char(10);
  if CommaDelimited then
    FieldSpacing:= ','
  else
    FieldSpacing:= char(9);
  j:= 0;
  with MyListView do
  begin
    for i:= 0 to Items.Count - 1 do
    if Items[i].Selected then
    begin
      k:= length(Items[i].Caption);
      Move(Items[i].Caption[1], Buffer[j], k);
      j:= j + k; {left justify}
      if not CommaDelimited then
      begin
        FillChar(Buffer[j], FieldLen[0] - k, ' ');
        j:= j + FieldLen[0] - k;
      end;

      p:= 0;
      for q:= 1 to MaxColumn do
        if View1.Items[q].Checked then
        begin
          Move(FieldSpacing[1], Buffer[j], length(FieldSpacing));
          j:= j + length(FieldSpacing);
          k:= length(Items[i].SubItems[p]);
          case q of
          1, 6 :
            begin
              Move(Items[i].SubItems[p][1], Buffer[j], k);
              j:= j + k; {left justify}
              if not CommaDelimited then
              begin
                FillChar(Buffer[j], FieldLen[q] - k, ' ');
                j:= j + FieldLen[q] - k;
              end;
            end;
          2..5 :
            begin
              if not CommaDelimited then
              begin
                FillChar(Buffer[j], FieldLen[q] - k, ' ');
                j:= j + FieldLen[q] - k;
              end;
              Move(Items[i].SubItems[p][1], Buffer[j], k);
              j:= j + k; {right justify}
            end;
          end;
          inc(p);
        end;
      Move(crlf[1], Buffer[j], 2);
      j:= j + 2;
    end;
    Buffer[j]:= Char(0);
  end;
end;

procedure TMainForm.CopyList(Dest : TDest);
var
  Text : TextFile;
begin
  if Busy then exit;
  Busy:= true;
  CalcBuffer;
  try
    GetMem(Buffer, BufferSize + 1);
    MakeBuffer;
    case Dest of
      ToClipboard : Clipboard.SetTextBuf(Buffer);

      ToPrinter :
        begin
          AssignPrn(Text);
          Rewrite(Text);
          Write(Text, Buffer);
          System.Close(Text);
        end;

      ToFile :
        try
          AssignFile(Text, SaveDialog1.FileName);
          Rewrite(Text);
          Write(Text, Buffer);
        finally
          System.Close(Text);
        end;
    end;
  finally
    FreeMem(Buffer, BufferSize + 1);
    Busy:= false;
  end;
end;

procedure TMainForm.ReadIni;
var
  i : integer;
  b : boolean;
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.INI')) do
  try
      with MainForm do
    begin
      Width:= ReadInteger('Setup', 'Width', 640);
      Height:= ReadInteger('Setup', 'Height', 340);
      Top:= ReadInteger('Setup', 'Top',
        (GetSystemMetrics(SM_CYSCREEN) - Height) div 2);
      Left:= ReadInteger('Setup', 'Left',
        (GetSystemMetrics(SM_CXSCREEN) - Width) div 2);
    end;

    with MyListView do
    begin
      Font.Name:= ReadString('Setup', 'FormatName', 'Arial');
      Font.Size:= ReadInteger('Setup', 'FormatSize', 10);
      b:= ReadBool('Setup', 'BoldFormat', false);
      if b then
        Font.Style:= [fsBold]
      else
        Font.Style:= [];
      b:= ReadBool('Setup', 'ItalicFormat', false);
      if b then
        Font.Style:= Font.Style + [fsItalic];

      for i:= 0 to MaxColumn do
        ColumnWidths[i]:= ReadInteger('Setup',
          'ColumnWidth' + IntToStr(i), DefaultWidths[i]);
    end;

    PrintName:= ReadString('Setup', 'PrintName', 'Courier New');
    PrintSize:= ReadInteger('Setup', 'PrintSize', 10);
    b:= ReadBool('Setup', 'BoldPrint', false);
    if b then
      PrintStyle:= [fsBold]
    else
      PrintStyle:= [];
    b:= ReadBool('Setup', 'ItalicPrint', false);
    if b then
      PrintStyle:= PrintStyle + [fsItalic];

    for i:= 0 to MaxColumn do
      View1.Items[i].Checked:= ReadBool('Setup',
        'ViewChecked' + IntToStr(i), DefaultChecked[i]);

    CommaDelimited:= ReadBool('Setup', 'CommaDelimited', false);
    with SaveDialog1 do
      if CommaDelimited then
        FilterIndex:= 2
      else
        FilterIndex:= 1;
  finally
    Free;
  end;
end;

procedure TMainForm.WriteIni;
var
  i : integer;
  b : boolean;
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.INI')) do
  try
    if WindowState <> wsMaximized then
    with MainForm do
    begin
      WriteInteger('Setup', 'Width', Width);
      WriteInteger('Setup', 'Height', Height);
      WriteInteger('Setup', 'Top', Top);
      WriteInteger('Setup', 'Left', Left);
    end;

    with MyListView do
    begin
      WriteString('Setup', 'FormatName', Font.Name);
      WriteInteger('Setup', 'FormatSize', Font.Size);
      if fsBold in Font.Style then
        b:= true
      else
        b:= false;
      WriteBool('Setup', 'BoldFormat', b);
      if fsItalic in Font.Style then
        b:= true
      else
        b:= false;
      WriteBool('Setup', 'ItalicFormat', b);

      for i:= 0 to MaxColumn do
        if ColumnWidths[i] > 0 then
          WriteInteger('Setup', 'ColumnWidth' + IntToStr(i),
            ColumnWidths[i]);
    end;

    WriteString('Setup', 'PrintName', PrintName);
    WriteInteger('Setup', 'PrintSize', PrintSize);
    if fsBold in PrintStyle then
      b:= true
    else
      b:= false;
    WriteBool('Setup', 'BoldPrint', b);
    if fsItalic in PrintStyle then
      b:= true
    else
      b:= false;
    WriteBool('Setup', 'ItalicPrint', b);

    for i:= 0 to MaxColumn do
      WriteBool('Setup', 'ViewChecked' + IntToStr(i),
        View1.Items[i].Checked);

    WriteBool('Setup', 'CommaDelimited', CommaDelimited);
  finally
    Free;
  end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i : integer;
begin
  SaveColumnWidths;
  WriteIni;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  InternalPasteMessage:= RegisterWindowMessage('FileGrab PasteMsg');
  MyListView:= TMyListView.Create(Self);
  with MyListView do
  begin
    Parent:= Self;
    Align:= alClient;
    MultiSelect:= true;
    ViewStyle:= vsReport;
    SmallImages:= ImageList1;
    DragAcceptFiles(Handle, true);
  end;
  ReadIni;
  ColumnsUpdate;
end;

procedure TMainForm.SaveColumnWidths;
var
  i, j : integer;
begin
  with MyListView do
  begin
    ColumnWidths[0]:= Columns[0].Width;
    j:= 1;
    for i:= 1 to MaxColumn do
      if View1.Items[i].Checked then
      begin
        ColumnWidths[i]:= Columns[j].Width;
        inc(j);
      end;
  end;
end;

procedure TMainForm.ColumnsUpdate;
var
  i, j : integer;
  NewItem : TListItem;
begin
  with MyListView do
  begin
    for i:= Items.Count - 1 downto 0 do
      TStringList(Items[i].Data).Free;
      
    Items.Clear;

    for i:= Columns.Count - 1 downto 0 do
      Columns[i].Free;

    Columns.Add;
    Columns[0].Caption:= Captions[0];
    Columns[0].Width:= ColumnWidths[0];

    for i:= 1 to MaxColumn do
      if View1.Items[i].Checked then
      begin
        Columns.Add;
        Columns[Columns.Count - 1].Caption:= Captions[i];
        Columns[Columns.Count - 1].Width:= ColumnWidths[i];
      end;

    for i:= 0 to FilenameList.Count - 1 do
    begin
      NewItem:= Items.Add;
      NewItem.ImageIndex:= integer(FilenameList.Objects[i]);
      if View1.Items[0].Checked then
        Items[i].Caption:= FilenameList[i]
      else
        Items[i].Caption:= ExtractFileName(FilenameList[i]);
      Items[i].Data:= TStringList.Create;
      TStringList(Items[i].Data).Add(FilenameList[i]);

      for j:= 1 to MaxColumn do
        if (View1.Items[j].Checked) and
           (i < ColumnLists[j].Count) then
          Items[i].SubItems.Add(ColumnLists[j].Strings[i]);
    end;

    AlphaSort;
  end;
  UpdateButtons;
end;

procedure TMainForm.ToggleChecked(Sender: TObject);
begin
  SaveColumnWidths;
  with Sender as TMenuItem do
    Checked:= not Checked;
  ColumnsUpdate;
end;

procedure TMainForm.AboutFileGrab1Click(Sender: TObject);
begin
  with TAboutBox.Create(Application) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMainForm.HelpTopics1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTENTS, 0);
end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.SaveAsSpeedButtonClick(Sender: TObject);
begin
  with SaveDialog1 do
    if Execute then
    begin
      CommaDelimited:= FilterIndex = 2;
      CopyList(ToFile);
    end;
end;

procedure TMainForm.PrintSpeedButtonClick(Sender: TObject);
begin
  if PrintDialog1.Execute then
  begin
    with Printer.Canvas.Font do
    begin
      Name:= PrintName;
      Size:= PrintSize;
      Style:= PrintStyle;
    end;
    CommaDelimited:= false;
    CopyList(ToPrinter);
  end;
end;

procedure TMainForm.CutSpeedButtonClick(Sender: TObject);
begin
  CopySpeedButtonClick(Sender);
  ClearSpeedButtonClick(Sender);
end;

procedure TMainForm.CopySpeedButtonClick(Sender: TObject);
begin
  CommaDelimited:= false;
  CopyList(ToClipboard);
end;

procedure TMainForm.ClearSpeedButtonClick(Sender: TObject);
var
  i, j, k : integer;
begin
  with MyListView do
  begin
    for i:= Items.Count - 1 downto 0 do
    if Items[i].Selected then
    begin
      k:= FilenameList.IndexOf(TStringList(Items[i].Data).Strings[0]);
      TStringList(Items[i].Data).Free;
      if k <> -1 then
      begin
        FilenameList.Delete(k);
        for j:= 1 to MaxColumn do
          if k < ColumnLists[j].Count then
            ColumnLists[j].Delete(k);
      end;
      Items[i].Delete;
    end;
  end;
  UpdateButtons;
end;

procedure TMainForm.SelectAllSpeedButtonClick(Sender: TObject);
var
  i : integer;
begin
  with MyListView do
    for i:= 0 to Items.Count - 1 do
      Items[i].Selected:= true;
  UpdateButtons;
end;

procedure TMainForm.ScreenFont1Click(Sender: TObject);
begin
  with FontDialog1 do
  begin
    Font.Name:= MyListView.Font.Name;
    Font.Size:= MyListView.Font.Size;
    Font.Style:= MyListView.Font.Style;
    if Execute then
    begin
      MyListView.Font.Name:= Font.Name;
      MyListView.Font.Size:= Font.Size;
      MyListView.Font.Style:= Font.Style;
    end;
  end;
end;

procedure TMainForm.PrintFont1Click(Sender: TObject);
begin
  with FontDialog1 do
  begin
    Font.Name:= PrintName;
    Font.Size:= PrintSize;
    Font.Style:= PrintStyle;
    if Execute then
    begin
      PrintName:= Font.Name;
      PrintSize:= Font.Size;
      PrintStyle:= Font.Style;
    end;
  end;
end;

procedure TMainForm.UpdateButtons;
begin
  with MyListView do
  begin
    Copy1.Enabled:= SelCount > 0;
    Clear1.Enabled:= SelCount > 0;
    CopySpeedButton.Enabled:= SelCount > 0;
    ClearSpeedButton.Enabled:= SelCount > 0;
    Cut1.Enabled:= SelCount > 0;
    CutSpeedButton.Enabled:= SelCount > 0;
    Print1.Enabled:= SelCount > 0;
    PrintSpeedButton.Enabled:= SelCount > 0;
    SelectAll1.Enabled:= Items.Count > 0;
    SaveAs1.Enabled:= Items.Count > 0;
    SaveAsSpeedButton.Enabled:= Items.Count > 0;
  end;
end;

procedure TMainForm.Edit1Click(Sender: TObject);
begin
  Paste1.Enabled:= Clipboard.HasFormat(CF_HDROP);
end;

procedure TMainForm.Paste1Click(Sender: TObject);
begin
  MyListView.Perform(InternalPasteMessage,
    Clipboard.GetAsHandle(CF_HDROP), 0);
end;

end.


